<?php

declare (strict_types=1);
/*
 * This file is part of PharIo\Manifest.
 *
 * (c) Arne Blankerts <arne@blankerts.de>, Sebastian Heuer <sebastian@phpeople.de>, Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\PharIo\Manifest;

use MailerLite_Woo\PharIo\Version\Exception as VersionException;
use MailerLite_Woo\PharIo\Version\Version;
use MailerLite_Woo\PharIo\Version\VersionConstraintParser;
class ManifestDocumentMapper
{
    public function map(\MailerLite_Woo\PharIo\Manifest\ManifestDocument $document) : \MailerLite_Woo\PharIo\Manifest\Manifest
    {
        try {
            $contains = $document->getContainsElement();
            $type = $this->mapType($contains);
            $copyright = $this->mapCopyright($document->getCopyrightElement());
            $requirements = $this->mapRequirements($document->getRequiresElement());
            $bundledComponents = $this->mapBundledComponents($document);
            return new \MailerLite_Woo\PharIo\Manifest\Manifest(new \MailerLite_Woo\PharIo\Manifest\ApplicationName($contains->getName()), new \MailerLite_Woo\PharIo\Version\Version($contains->getVersion()), $type, $copyright, $requirements, $bundledComponents);
        } catch (\MailerLite_Woo\PharIo\Version\Exception $e) {
            throw new \MailerLite_Woo\PharIo\Manifest\ManifestDocumentMapperException($e->getMessage(), (int) $e->getCode(), $e);
        } catch (\MailerLite_Woo\PharIo\Manifest\Exception $e) {
            throw new \MailerLite_Woo\PharIo\Manifest\ManifestDocumentMapperException($e->getMessage(), (int) $e->getCode(), $e);
        }
    }
    private function mapType(\MailerLite_Woo\PharIo\Manifest\ContainsElement $contains) : \MailerLite_Woo\PharIo\Manifest\Type
    {
        switch ($contains->getType()) {
            case 'application':
                return \MailerLite_Woo\PharIo\Manifest\Type::application();
            case 'library':
                return \MailerLite_Woo\PharIo\Manifest\Type::library();
            case 'extension':
                return $this->mapExtension($contains->getExtensionElement());
        }
        throw new \MailerLite_Woo\PharIo\Manifest\ManifestDocumentMapperException(\sprintf('Unsupported type %s', $contains->getType()));
    }
    private function mapCopyright(\MailerLite_Woo\PharIo\Manifest\CopyrightElement $copyright) : \MailerLite_Woo\PharIo\Manifest\CopyrightInformation
    {
        $authors = new \MailerLite_Woo\PharIo\Manifest\AuthorCollection();
        foreach ($copyright->getAuthorElements() as $authorElement) {
            $authors->add(new \MailerLite_Woo\PharIo\Manifest\Author($authorElement->getName(), new \MailerLite_Woo\PharIo\Manifest\Email($authorElement->getEmail())));
        }
        $licenseElement = $copyright->getLicenseElement();
        $license = new \MailerLite_Woo\PharIo\Manifest\License($licenseElement->getType(), new \MailerLite_Woo\PharIo\Manifest\Url($licenseElement->getUrl()));
        return new \MailerLite_Woo\PharIo\Manifest\CopyrightInformation($authors, $license);
    }
    private function mapRequirements(\MailerLite_Woo\PharIo\Manifest\RequiresElement $requires) : \MailerLite_Woo\PharIo\Manifest\RequirementCollection
    {
        $collection = new \MailerLite_Woo\PharIo\Manifest\RequirementCollection();
        $phpElement = $requires->getPHPElement();
        $parser = new \MailerLite_Woo\PharIo\Version\VersionConstraintParser();
        try {
            $versionConstraint = $parser->parse($phpElement->getVersion());
        } catch (\MailerLite_Woo\PharIo\Version\Exception $e) {
            throw new \MailerLite_Woo\PharIo\Manifest\ManifestDocumentMapperException(\sprintf('Unsupported version constraint - %s', $e->getMessage()), (int) $e->getCode(), $e);
        }
        $collection->add(new \MailerLite_Woo\PharIo\Manifest\PhpVersionRequirement($versionConstraint));
        if (!$phpElement->hasExtElements()) {
            return $collection;
        }
        foreach ($phpElement->getExtElements() as $extElement) {
            $collection->add(new \MailerLite_Woo\PharIo\Manifest\PhpExtensionRequirement($extElement->getName()));
        }
        return $collection;
    }
    private function mapBundledComponents(\MailerLite_Woo\PharIo\Manifest\ManifestDocument $document) : \MailerLite_Woo\PharIo\Manifest\BundledComponentCollection
    {
        $collection = new \MailerLite_Woo\PharIo\Manifest\BundledComponentCollection();
        if (!$document->hasBundlesElement()) {
            return $collection;
        }
        foreach ($document->getBundlesElement()->getComponentElements() as $componentElement) {
            $collection->add(new \MailerLite_Woo\PharIo\Manifest\BundledComponent($componentElement->getName(), new \MailerLite_Woo\PharIo\Version\Version($componentElement->getVersion())));
        }
        return $collection;
    }
    private function mapExtension(\MailerLite_Woo\PharIo\Manifest\ExtensionElement $extension) : \MailerLite_Woo\PharIo\Manifest\Extension
    {
        try {
            $versionConstraint = (new \MailerLite_Woo\PharIo\Version\VersionConstraintParser())->parse($extension->getCompatible());
            return \MailerLite_Woo\PharIo\Manifest\Type::extension(new \MailerLite_Woo\PharIo\Manifest\ApplicationName($extension->getFor()), $versionConstraint);
        } catch (\MailerLite_Woo\PharIo\Version\Exception $e) {
            throw new \MailerLite_Woo\PharIo\Manifest\ManifestDocumentMapperException(\sprintf('Unsupported version constraint - %s', $e->getMessage()), (int) $e->getCode(), $e);
        }
    }
}
