<?php

/*
 * This file is part of the Prophecy.
 * (c) Konstantin Kudryashov <ever.zet@gmail.com>
 *     Marcello Duarte <marcello.duarte@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\Prophecy;

use MailerLite_Woo\Prophecy\Doubler\CachedDoubler;
use MailerLite_Woo\Prophecy\Doubler\Doubler;
use MailerLite_Woo\Prophecy\Doubler\LazyDouble;
use MailerLite_Woo\Prophecy\Doubler\ClassPatch;
use MailerLite_Woo\Prophecy\Prophecy\ObjectProphecy;
use MailerLite_Woo\Prophecy\Prophecy\RevealerInterface;
use MailerLite_Woo\Prophecy\Prophecy\Revealer;
use MailerLite_Woo\Prophecy\Call\CallCenter;
use MailerLite_Woo\Prophecy\Util\StringUtil;
use MailerLite_Woo\Prophecy\Exception\Prediction\PredictionException;
use MailerLite_Woo\Prophecy\Exception\Prediction\AggregateException;
/**
 * Prophet creates prophecies.
 *
 * @author Konstantin Kudryashov <ever.zet@gmail.com>
 */
class Prophet
{
    private $doubler;
    private $revealer;
    private $util;
    /**
     * @var ObjectProphecy[]
     */
    private $prophecies = array();
    /**
     * Initializes Prophet.
     *
     * @param null|Doubler           $doubler
     * @param null|RevealerInterface $revealer
     * @param null|StringUtil        $util
     */
    public function __construct(\MailerLite_Woo\Prophecy\Doubler\Doubler $doubler = null, \MailerLite_Woo\Prophecy\Prophecy\RevealerInterface $revealer = null, \MailerLite_Woo\Prophecy\Util\StringUtil $util = null)
    {
        if (null === $doubler) {
            $doubler = new \MailerLite_Woo\Prophecy\Doubler\CachedDoubler();
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\SplFileInfoPatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\TraversablePatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\ThrowablePatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\DisableConstructorPatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\ProphecySubjectPatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\ReflectionClassNewInstancePatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\HhvmExceptionPatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\MagicCallPatch());
            $doubler->registerClassPatch(new \MailerLite_Woo\Prophecy\Doubler\ClassPatch\KeywordPatch());
        }
        $this->doubler = $doubler;
        $this->revealer = $revealer ?: new \MailerLite_Woo\Prophecy\Prophecy\Revealer();
        $this->util = $util ?: new \MailerLite_Woo\Prophecy\Util\StringUtil();
    }
    /**
     * Creates new object prophecy.
     *
     * @param null|string $classOrInterface Class or interface name
     *
     * @return ObjectProphecy
     */
    public function prophesize($classOrInterface = null)
    {
        $this->prophecies[] = $prophecy = new \MailerLite_Woo\Prophecy\Prophecy\ObjectProphecy(new \MailerLite_Woo\Prophecy\Doubler\LazyDouble($this->doubler), new \MailerLite_Woo\Prophecy\Call\CallCenter($this->util), $this->revealer);
        if ($classOrInterface && \class_exists($classOrInterface)) {
            return $prophecy->willExtend($classOrInterface);
        }
        if ($classOrInterface && \interface_exists($classOrInterface)) {
            return $prophecy->willImplement($classOrInterface);
        }
        return $prophecy;
    }
    /**
     * Returns all created object prophecies.
     *
     * @return ObjectProphecy[]
     */
    public function getProphecies()
    {
        return $this->prophecies;
    }
    /**
     * Returns Doubler instance assigned to this Prophet.
     *
     * @return Doubler
     */
    public function getDoubler()
    {
        return $this->doubler;
    }
    /**
     * Checks all predictions defined by prophecies of this Prophet.
     *
     * @throws Exception\Prediction\AggregateException If any prediction fails
     */
    public function checkPredictions()
    {
        $exception = new \MailerLite_Woo\Prophecy\Exception\Prediction\AggregateException("Some predictions failed:\n");
        foreach ($this->prophecies as $prophecy) {
            try {
                $prophecy->checkProphecyMethodsPredictions();
            } catch (\MailerLite_Woo\Prophecy\Exception\Prediction\PredictionException $e) {
                $exception->append($e);
            }
        }
        if (\count($exception->getExceptions())) {
            throw $exception;
        }
    }
}
