<?php

declare (strict_types=1);
/*
 * This file is part of phpunit/php-code-coverage.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis;

use function implode;
use function rtrim;
use function trim;
use MailerLite_Woo\PhpParser\Node;
use MailerLite_Woo\PhpParser\Node\Identifier;
use MailerLite_Woo\PhpParser\Node\Name;
use MailerLite_Woo\PhpParser\Node\NullableType;
use MailerLite_Woo\PhpParser\Node\Stmt\Class_;
use MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod;
use MailerLite_Woo\PhpParser\Node\Stmt\Function_;
use MailerLite_Woo\PhpParser\Node\Stmt\Interface_;
use MailerLite_Woo\PhpParser\Node\Stmt\Trait_;
use MailerLite_Woo\PhpParser\Node\UnionType;
use MailerLite_Woo\PhpParser\NodeTraverser;
use MailerLite_Woo\PhpParser\NodeVisitorAbstract;
use MailerLite_Woo\SebastianBergmann\Complexity\CyclomaticComplexityCalculatingVisitor;
/**
 * @internal This class is not covered by the backward compatibility promise for phpunit/php-code-coverage
 */
final class CodeUnitFindingVisitor extends \MailerLite_Woo\PhpParser\NodeVisitorAbstract
{
    /**
     * @var array
     */
    private $classes = [];
    /**
     * @var array
     */
    private $traits = [];
    /**
     * @var array
     */
    private $functions = [];
    public function enterNode(\MailerLite_Woo\PhpParser\Node $node)
    {
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_) {
            if ($node->isAnonymous()) {
                return;
            }
            $this->processClass($node);
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Trait_) {
            $this->processTrait($node);
        }
        if (!$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_) {
            return null;
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod) {
            $parentNode = $node->getAttribute('parent');
            if ($parentNode instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ && $parentNode->isAnonymous()) {
                return;
            }
            $this->processMethod($node);
            return;
        }
        $this->processFunction($node);
    }
    public function classes() : array
    {
        return $this->classes;
    }
    public function traits() : array
    {
        return $this->traits;
    }
    public function functions() : array
    {
        return $this->functions;
    }
    /**
     * @psalm-param ClassMethod|Function_ $node
     */
    private function cyclomaticComplexity(\MailerLite_Woo\PhpParser\Node $node) : int
    {
        \assert($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod || $node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_);
        $nodes = $node->getStmts();
        if ($nodes === null) {
            return 0;
        }
        $traverser = new \MailerLite_Woo\PhpParser\NodeTraverser();
        $cyclomaticComplexityCalculatingVisitor = new \MailerLite_Woo\SebastianBergmann\Complexity\CyclomaticComplexityCalculatingVisitor();
        $traverser->addVisitor($cyclomaticComplexityCalculatingVisitor);
        /* @noinspection UnusedFunctionResultInspection */
        $traverser->traverse($nodes);
        return $cyclomaticComplexityCalculatingVisitor->cyclomaticComplexity();
    }
    /**
     * @psalm-param ClassMethod|Function_ $node
     */
    private function signature(\MailerLite_Woo\PhpParser\Node $node) : string
    {
        \assert($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod || $node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_);
        $signature = ($node->returnsByRef() ? '&' : '') . $node->name->toString() . '(';
        $parameters = [];
        foreach ($node->getParams() as $parameter) {
            \assert(isset($parameter->var->name));
            $parameterAsString = '';
            if ($parameter->type !== null) {
                $parameterAsString = $this->type($parameter->type) . ' ';
            }
            $parameterAsString .= '$' . $parameter->var->name;
            /* @todo Handle default values */
            $parameters[] = $parameterAsString;
        }
        $signature .= \implode(', ', $parameters) . ')';
        $returnType = $node->getReturnType();
        if ($returnType !== null) {
            $signature .= ': ' . $this->type($returnType);
        }
        return $signature;
    }
    /**
     * @psalm-param Identifier|Name|NullableType|UnionType $type
     */
    private function type(\MailerLite_Woo\PhpParser\Node $type) : string
    {
        \assert($type instanceof \MailerLite_Woo\PhpParser\Node\Identifier || $type instanceof \MailerLite_Woo\PhpParser\Node\Name || $type instanceof \MailerLite_Woo\PhpParser\Node\NullableType || $type instanceof \MailerLite_Woo\PhpParser\Node\UnionType);
        if ($type instanceof \MailerLite_Woo\PhpParser\Node\NullableType) {
            return '?' . $type->type;
        }
        if ($type instanceof \MailerLite_Woo\PhpParser\Node\UnionType) {
            $types = [];
            foreach ($type->types as $_type) {
                $types[] = $_type->toString();
            }
            return \implode('|', $types);
        }
        return $type->toString();
    }
    private function visibility(\MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod $node) : string
    {
        if ($node->isPrivate()) {
            return 'private';
        }
        if ($node->isProtected()) {
            return 'protected';
        }
        return 'public';
    }
    private function processClass(\MailerLite_Woo\PhpParser\Node\Stmt\Class_ $node) : void
    {
        $name = $node->name->toString();
        $namespacedName = $node->namespacedName->toString();
        $this->classes[$namespacedName] = ['name' => $name, 'namespacedName' => $namespacedName, 'namespace' => $this->namespace($namespacedName, $name), 'startLine' => $node->getStartLine(), 'endLine' => $node->getEndLine(), 'methods' => []];
    }
    private function processTrait(\MailerLite_Woo\PhpParser\Node\Stmt\Trait_ $node) : void
    {
        $name = $node->name->toString();
        $namespacedName = $node->namespacedName->toString();
        $this->traits[$namespacedName] = ['name' => $name, 'namespacedName' => $namespacedName, 'namespace' => $this->namespace($namespacedName, $name), 'startLine' => $node->getStartLine(), 'endLine' => $node->getEndLine(), 'methods' => []];
    }
    private function processMethod(\MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod $node) : void
    {
        $parentNode = $node->getAttribute('parent');
        if ($parentNode instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Interface_) {
            return;
        }
        \assert($parentNode instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ || $parentNode instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Trait_);
        \assert(isset($parentNode->name));
        \assert(isset($parentNode->namespacedName));
        \assert($parentNode->namespacedName instanceof \MailerLite_Woo\PhpParser\Node\Name);
        $parentName = $parentNode->name->toString();
        $parentNamespacedName = $parentNode->namespacedName->toString();
        if ($parentNode instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_) {
            $storage =& $this->classes;
        } else {
            $storage =& $this->traits;
        }
        if (!isset($storage[$parentNamespacedName])) {
            $storage[$parentNamespacedName] = ['name' => $parentName, 'namespacedName' => $parentNamespacedName, 'namespace' => $this->namespace($parentNamespacedName, $parentName), 'startLine' => $parentNode->getStartLine(), 'endLine' => $parentNode->getEndLine(), 'methods' => []];
        }
        $storage[$parentNamespacedName]['methods'][$node->name->toString()] = ['methodName' => $node->name->toString(), 'signature' => $this->signature($node), 'visibility' => $this->visibility($node), 'startLine' => $node->getStartLine(), 'endLine' => $node->getEndLine(), 'ccn' => $this->cyclomaticComplexity($node)];
    }
    private function processFunction(\MailerLite_Woo\PhpParser\Node\Stmt\Function_ $node) : void
    {
        \assert(isset($node->name));
        \assert(isset($node->namespacedName));
        \assert($node->namespacedName instanceof \MailerLite_Woo\PhpParser\Node\Name);
        $name = $node->name->toString();
        $namespacedName = $node->namespacedName->toString();
        $this->functions[$namespacedName] = ['name' => $name, 'namespacedName' => $namespacedName, 'namespace' => $this->namespace($namespacedName, $name), 'signature' => $this->signature($node), 'startLine' => $node->getStartLine(), 'endLine' => $node->getEndLine(), 'ccn' => $this->cyclomaticComplexity($node)];
    }
    private function namespace(string $namespacedName, string $name) : string
    {
        return \trim(\rtrim($namespacedName, $name), '\\');
    }
}
