<?php

declare (strict_types=1);
/*
 * This file is part of phpunit/php-code-coverage.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis;

use function array_merge;
use function range;
use function strpos;
use MailerLite_Woo\PhpParser\Node;
use MailerLite_Woo\PhpParser\Node\Stmt\Class_;
use MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod;
use MailerLite_Woo\PhpParser\Node\Stmt\Function_;
use MailerLite_Woo\PhpParser\Node\Stmt\Interface_;
use MailerLite_Woo\PhpParser\Node\Stmt\Trait_;
use MailerLite_Woo\PhpParser\NodeTraverser;
use MailerLite_Woo\PhpParser\NodeVisitorAbstract;
/**
 * @internal This class is not covered by the backward compatibility promise for phpunit/php-code-coverage
 */
final class IgnoredLinesFindingVisitor extends \MailerLite_Woo\PhpParser\NodeVisitorAbstract
{
    /**
     * @psalm-var list<int>
     */
    private $ignoredLines = [];
    /**
     * @var bool
     */
    private $useAnnotationsForIgnoringCode;
    /**
     * @var bool
     */
    private $ignoreDeprecated;
    public function __construct(bool $useAnnotationsForIgnoringCode, bool $ignoreDeprecated)
    {
        $this->useAnnotationsForIgnoringCode = $useAnnotationsForIgnoringCode;
        $this->ignoreDeprecated = $ignoreDeprecated;
    }
    public function enterNode(\MailerLite_Woo\PhpParser\Node $node) : ?int
    {
        if (!$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Trait_ && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Interface_ && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_) {
            return null;
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ && $node->isAnonymous()) {
            return null;
        }
        // Workaround for https://bugs.xdebug.org/view.php?id=1798
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ || $node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Trait_ || $node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Interface_) {
            $this->ignoredLines[] = $node->getStartLine();
        }
        if (!$this->useAnnotationsForIgnoringCode) {
            return null;
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Interface_) {
            return null;
        }
        $docComment = $node->getDocComment();
        if ($docComment === null) {
            return null;
        }
        if (\strpos($docComment->getText(), '@codeCoverageIgnore') !== \false) {
            $this->ignoredLines = \array_merge($this->ignoredLines, \range($node->getStartLine(), $node->getEndLine()));
        }
        if ($this->ignoreDeprecated && \strpos($docComment->getText(), '@deprecated') !== \false) {
            $this->ignoredLines = \array_merge($this->ignoredLines, \range($node->getStartLine(), $node->getEndLine()));
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod || $node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_) {
            return \MailerLite_Woo\PhpParser\NodeTraverser::DONT_TRAVERSE_CHILDREN;
        }
        return null;
    }
    /**
     * @psalm-return list<int>
     */
    public function ignoredLines() : array
    {
        return $this->ignoredLines;
    }
}
