<?php

declare (strict_types=1);
/*
 * This file is part of phpunit/php-code-coverage.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis;

use function array_unique;
use function assert;
use function file_get_contents;
use function is_array;
use function sprintf;
use function substr_count;
use function token_get_all;
use function trim;
use MailerLite_Woo\PhpParser\Error;
use MailerLite_Woo\PhpParser\Lexer;
use MailerLite_Woo\PhpParser\NodeTraverser;
use MailerLite_Woo\PhpParser\NodeVisitor\NameResolver;
use MailerLite_Woo\PhpParser\NodeVisitor\ParentConnectingVisitor;
use MailerLite_Woo\PhpParser\ParserFactory;
use MailerLite_Woo\SebastianBergmann\CodeCoverage\ParserException;
use MailerLite_Woo\SebastianBergmann\LinesOfCode\LineCountingVisitor;
use MailerLite_Woo\SebastianBergmann\LinesOfCode\LinesOfCode;
/**
 * @internal This class is not covered by the backward compatibility promise for phpunit/php-code-coverage
 */
final class ParsingCoveredFileAnalyser implements \MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis\CoveredFileAnalyser
{
    /**
     * @var array
     */
    private $classes = [];
    /**
     * @var array
     */
    private $traits = [];
    /**
     * @var array
     */
    private $functions = [];
    /**
     * @var LinesOfCode[]
     */
    private $linesOfCode = [];
    /**
     * @var array
     */
    private $ignoredLines = [];
    /**
     * @var bool
     */
    private $useAnnotationsForIgnoringCode;
    /**
     * @var bool
     */
    private $ignoreDeprecatedCode;
    public function __construct(bool $useAnnotationsForIgnoringCode, bool $ignoreDeprecatedCode)
    {
        $this->useAnnotationsForIgnoringCode = $useAnnotationsForIgnoringCode;
        $this->ignoreDeprecatedCode = $ignoreDeprecatedCode;
    }
    public function classesIn(string $filename) : array
    {
        $this->analyse($filename);
        return $this->classes[$filename];
    }
    public function traitsIn(string $filename) : array
    {
        $this->analyse($filename);
        return $this->traits[$filename];
    }
    public function functionsIn(string $filename) : array
    {
        $this->analyse($filename);
        return $this->functions[$filename];
    }
    public function linesOfCodeFor(string $filename) : \MailerLite_Woo\SebastianBergmann\LinesOfCode\LinesOfCode
    {
        $this->analyse($filename);
        return $this->linesOfCode[$filename];
    }
    public function ignoredLinesFor(string $filename) : array
    {
        $this->analyse($filename);
        return $this->ignoredLines[$filename];
    }
    /**
     * @throws ParserException
     */
    private function analyse(string $filename) : void
    {
        if (isset($this->classes[$filename])) {
            return;
        }
        $source = \file_get_contents($filename);
        $linesOfCode = \substr_count($source, "\n");
        if ($linesOfCode === 0 && !empty($source)) {
            $linesOfCode = 1;
        }
        $parser = (new \MailerLite_Woo\PhpParser\ParserFactory())->create(\MailerLite_Woo\PhpParser\ParserFactory::PREFER_PHP7, new \MailerLite_Woo\PhpParser\Lexer());
        try {
            $nodes = $parser->parse($source);
            \assert($nodes !== null);
            $traverser = new \MailerLite_Woo\PhpParser\NodeTraverser();
            $codeUnitFindingVisitor = new \MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis\CodeUnitFindingVisitor();
            $lineCountingVisitor = new \MailerLite_Woo\SebastianBergmann\LinesOfCode\LineCountingVisitor($linesOfCode);
            $ignoredLinesFindingVisitor = new \MailerLite_Woo\SebastianBergmann\CodeCoverage\StaticAnalysis\IgnoredLinesFindingVisitor($this->useAnnotationsForIgnoringCode, $this->ignoreDeprecatedCode);
            $traverser->addVisitor(new \MailerLite_Woo\PhpParser\NodeVisitor\NameResolver());
            $traverser->addVisitor(new \MailerLite_Woo\PhpParser\NodeVisitor\ParentConnectingVisitor());
            $traverser->addVisitor($codeUnitFindingVisitor);
            $traverser->addVisitor($lineCountingVisitor);
            $traverser->addVisitor($ignoredLinesFindingVisitor);
            /* @noinspection UnusedFunctionResultInspection */
            $traverser->traverse($nodes);
            // @codeCoverageIgnoreStart
        } catch (\MailerLite_Woo\PhpParser\Error $error) {
            throw new \MailerLite_Woo\SebastianBergmann\CodeCoverage\ParserException(\sprintf('Cannot parse %s: %s', $filename, $error->getMessage()), (int) $error->getCode(), $error);
        }
        // @codeCoverageIgnoreEnd
        $this->classes[$filename] = $codeUnitFindingVisitor->classes();
        $this->traits[$filename] = $codeUnitFindingVisitor->traits();
        $this->functions[$filename] = $codeUnitFindingVisitor->functions();
        $this->linesOfCode[$filename] = $lineCountingVisitor->result();
        $this->ignoredLines[$filename] = [];
        $this->findLinesIgnoredByLineBasedAnnotations($filename, $source, $this->useAnnotationsForIgnoringCode);
        $this->ignoredLines[$filename] = \array_unique(\array_merge($this->ignoredLines[$filename], $ignoredLinesFindingVisitor->ignoredLines()));
        \sort($this->ignoredLines[$filename]);
    }
    private function findLinesIgnoredByLineBasedAnnotations(string $filename, string $source, bool $useAnnotationsForIgnoringCode) : void
    {
        $ignore = \false;
        $stop = \false;
        foreach (\token_get_all($source) as $token) {
            if (!\is_array($token)) {
                continue;
            }
            switch ($token[0]) {
                case \T_COMMENT:
                case \T_DOC_COMMENT:
                    if (!$useAnnotationsForIgnoringCode) {
                        break;
                    }
                    $comment = \trim($token[1]);
                    if ($comment === '// @codeCoverageIgnore' || $comment === '//@codeCoverageIgnore') {
                        $ignore = \true;
                        $stop = \true;
                    } elseif ($comment === '// @codeCoverageIgnoreStart' || $comment === '//@codeCoverageIgnoreStart') {
                        $ignore = \true;
                    } elseif ($comment === '// @codeCoverageIgnoreEnd' || $comment === '//@codeCoverageIgnoreEnd') {
                        $stop = \true;
                    }
                    break;
            }
            if ($ignore) {
                $this->ignoredLines[$filename][] = $token[2];
                if ($stop) {
                    $ignore = \false;
                    $stop = \false;
                }
            }
        }
    }
}
