<?php

declare (strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\PHPUnit\Runner;

use MailerLite_Woo\PHPUnit\Framework\AssertionFailedError;
use MailerLite_Woo\PHPUnit\Framework\Test;
use MailerLite_Woo\PHPUnit\Framework\TestListener;
use MailerLite_Woo\PHPUnit\Framework\TestSuite;
use MailerLite_Woo\PHPUnit\Framework\Warning;
use MailerLite_Woo\PHPUnit\Util\Test as TestUtil;
use Throwable;
/**
 * @internal This class is not covered by the backward compatibility promise for PHPUnit
 */
final class TestListenerAdapter implements \MailerLite_Woo\PHPUnit\Framework\TestListener
{
    /**
     * @var TestHook[]
     */
    private $hooks = [];
    /**
     * @var bool
     */
    private $lastTestWasNotSuccessful;
    public function add(\MailerLite_Woo\PHPUnit\Runner\TestHook $hook) : void
    {
        $this->hooks[] = $hook;
    }
    public function startTest(\MailerLite_Woo\PHPUnit\Framework\Test $test) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\BeforeTestHook) {
                $hook->executeBeforeTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test));
            }
        }
        $this->lastTestWasNotSuccessful = \false;
    }
    public function addError(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterTestErrorHook) {
                $hook->executeAfterTestError(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $t->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function addWarning(\MailerLite_Woo\PHPUnit\Framework\Test $test, \MailerLite_Woo\PHPUnit\Framework\Warning $e, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterTestWarningHook) {
                $hook->executeAfterTestWarning(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $e->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function addFailure(\MailerLite_Woo\PHPUnit\Framework\Test $test, \MailerLite_Woo\PHPUnit\Framework\AssertionFailedError $e, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterTestFailureHook) {
                $hook->executeAfterTestFailure(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $e->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function addIncompleteTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterIncompleteTestHook) {
                $hook->executeAfterIncompleteTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $t->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function addRiskyTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterRiskyTestHook) {
                $hook->executeAfterRiskyTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $t->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function addSkippedTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterSkippedTestHook) {
                $hook->executeAfterSkippedTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $t->getMessage(), $time);
            }
        }
        $this->lastTestWasNotSuccessful = \true;
    }
    public function endTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, float $time) : void
    {
        if (!$this->lastTestWasNotSuccessful) {
            foreach ($this->hooks as $hook) {
                if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterSuccessfulTestHook) {
                    $hook->executeAfterSuccessfulTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $time);
                }
            }
        }
        foreach ($this->hooks as $hook) {
            if ($hook instanceof \MailerLite_Woo\PHPUnit\Runner\AfterTestHook) {
                $hook->executeAfterTest(\MailerLite_Woo\PHPUnit\Util\Test::describeAsString($test), $time);
            }
        }
    }
    public function startTestSuite(\MailerLite_Woo\PHPUnit\Framework\TestSuite $suite) : void
    {
    }
    public function endTestSuite(\MailerLite_Woo\PHPUnit\Framework\TestSuite $suite) : void
    {
    }
}
