<?php

declare (strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\PHPUnit\Util;

use const DIRECTORY_SEPARATOR;
use function class_exists;
use function defined;
use function dirname;
use function is_dir;
use function realpath;
use function sprintf;
use function strpos;
use function sys_get_temp_dir;
use MailerLite_Woo\Composer\Autoload\ClassLoader;
use MailerLite_Woo\DeepCopy\DeepCopy;
use MailerLite_Woo\Doctrine\Instantiator\Instantiator;
use MailerLite_Woo\PharIo\Manifest\Manifest;
use MailerLite_Woo\PharIo\Version\Version as PharIoVersion;
use MailerLite_Woo\phpDocumentor\Reflection\DocBlock;
use MailerLite_Woo\phpDocumentor\Reflection\Project;
use MailerLite_Woo\phpDocumentor\Reflection\Type;
use MailerLite_Woo\PhpParser\Parser;
use MailerLite_Woo\PHPUnit\Framework\TestCase;
use MailerLite_Woo\Prophecy\Prophet;
use ReflectionClass;
use ReflectionException;
use MailerLite_Woo\SebastianBergmann\CliParser\Parser as CliParser;
use MailerLite_Woo\SebastianBergmann\CodeCoverage\CodeCoverage;
use MailerLite_Woo\SebastianBergmann\CodeUnit\CodeUnit;
use MailerLite_Woo\SebastianBergmann\CodeUnitReverseLookup\Wizard;
use MailerLite_Woo\SebastianBergmann\Comparator\Comparator;
use MailerLite_Woo\SebastianBergmann\Complexity\Calculator;
use MailerLite_Woo\SebastianBergmann\Diff\Diff;
use MailerLite_Woo\SebastianBergmann\Environment\Runtime;
use MailerLite_Woo\SebastianBergmann\Exporter\Exporter;
use MailerLite_Woo\SebastianBergmann\FileIterator\Facade as FileIteratorFacade;
use MailerLite_Woo\SebastianBergmann\GlobalState\Snapshot;
use MailerLite_Woo\SebastianBergmann\Invoker\Invoker;
use MailerLite_Woo\SebastianBergmann\LinesOfCode\Counter;
use MailerLite_Woo\SebastianBergmann\ObjectEnumerator\Enumerator;
use MailerLite_Woo\SebastianBergmann\RecursionContext\Context;
use MailerLite_Woo\SebastianBergmann\ResourceOperations\ResourceOperations;
use MailerLite_Woo\SebastianBergmann\Template\Template;
use MailerLite_Woo\SebastianBergmann\Timer\Timer;
use MailerLite_Woo\SebastianBergmann\Type\TypeName;
use MailerLite_Woo\SebastianBergmann\Version;
use MailerLite_Woo\Symfony\Polyfill\Ctype\Ctype;
use MailerLite_Woo\TheSeer\Tokenizer\Tokenizer;
use MailerLite_Woo\Webmozart\Assert\Assert;
/**
 * @no-named-arguments Parameter names are not covered by the backward compatibility promise for PHPUnit
 */
final class ExcludeList
{
    /**
     * @var array<string,int>
     */
    private const EXCLUDED_CLASS_NAMES = [
        // composer
        \MailerLite_Woo\Composer\Autoload\ClassLoader::class => 1,
        // doctrine/instantiator
        \MailerLite_Woo\Doctrine\Instantiator\Instantiator::class => 1,
        // myclabs/deepcopy
        \MailerLite_Woo\DeepCopy\DeepCopy::class => 1,
        // nikic/php-parser
        \MailerLite_Woo\PhpParser\Parser::class => 1,
        // phar-io/manifest
        \MailerLite_Woo\PharIo\Manifest\Manifest::class => 1,
        // phar-io/version
        \MailerLite_Woo\PharIo\Version\Version::class => 1,
        // phpdocumentor/reflection-common
        \MailerLite_Woo\phpDocumentor\Reflection\Project::class => 1,
        // phpdocumentor/reflection-docblock
        \MailerLite_Woo\phpDocumentor\Reflection\DocBlock::class => 1,
        // phpdocumentor/type-resolver
        \MailerLite_Woo\phpDocumentor\Reflection\Type::class => 1,
        // phpspec/prophecy
        \MailerLite_Woo\Prophecy\Prophet::class => 1,
        // phpunit/phpunit
        \MailerLite_Woo\PHPUnit\Framework\TestCase::class => 2,
        // phpunit/php-code-coverage
        \MailerLite_Woo\SebastianBergmann\CodeCoverage\CodeCoverage::class => 1,
        // phpunit/php-file-iterator
        \MailerLite_Woo\SebastianBergmann\FileIterator\Facade::class => 1,
        // phpunit/php-invoker
        \MailerLite_Woo\SebastianBergmann\Invoker\Invoker::class => 1,
        // phpunit/php-text-template
        \MailerLite_Woo\SebastianBergmann\Template\Template::class => 1,
        // phpunit/php-timer
        \MailerLite_Woo\SebastianBergmann\Timer\Timer::class => 1,
        // sebastian/cli-parser
        \MailerLite_Woo\SebastianBergmann\CliParser\Parser::class => 1,
        // sebastian/code-unit
        \MailerLite_Woo\SebastianBergmann\CodeUnit\CodeUnit::class => 1,
        // sebastian/code-unit-reverse-lookup
        \MailerLite_Woo\SebastianBergmann\CodeUnitReverseLookup\Wizard::class => 1,
        // sebastian/comparator
        \MailerLite_Woo\SebastianBergmann\Comparator\Comparator::class => 1,
        // sebastian/complexity
        \MailerLite_Woo\SebastianBergmann\Complexity\Calculator::class => 1,
        // sebastian/diff
        \MailerLite_Woo\SebastianBergmann\Diff\Diff::class => 1,
        // sebastian/environment
        \MailerLite_Woo\SebastianBergmann\Environment\Runtime::class => 1,
        // sebastian/exporter
        \MailerLite_Woo\SebastianBergmann\Exporter\Exporter::class => 1,
        // sebastian/global-state
        \MailerLite_Woo\SebastianBergmann\GlobalState\Snapshot::class => 1,
        // sebastian/lines-of-code
        \MailerLite_Woo\SebastianBergmann\LinesOfCode\Counter::class => 1,
        // sebastian/object-enumerator
        \MailerLite_Woo\SebastianBergmann\ObjectEnumerator\Enumerator::class => 1,
        // sebastian/recursion-context
        \MailerLite_Woo\SebastianBergmann\RecursionContext\Context::class => 1,
        // sebastian/resource-operations
        \MailerLite_Woo\SebastianBergmann\ResourceOperations\ResourceOperations::class => 1,
        // sebastian/type
        \MailerLite_Woo\SebastianBergmann\Type\TypeName::class => 1,
        // sebastian/version
        \MailerLite_Woo\SebastianBergmann\Version::class => 1,
        // symfony/polyfill-ctype
        \MailerLite_Woo\Symfony\Polyfill\Ctype\Ctype::class => 1,
        // theseer/tokenizer
        \MailerLite_Woo\TheSeer\Tokenizer\Tokenizer::class => 1,
        // webmozart/assert
        \MailerLite_Woo\Webmozart\Assert\Assert::class => 1,
    ];
    /**
     * @var string[]
     */
    private static $directories;
    public static function addDirectory(string $directory) : void
    {
        if (!\is_dir($directory)) {
            throw new \MailerLite_Woo\PHPUnit\Util\Exception(\sprintf('"%s" is not a directory', $directory));
        }
        self::$directories[] = \realpath($directory);
    }
    /**
     * @throws Exception
     *
     * @return string[]
     */
    public function getExcludedDirectories() : array
    {
        $this->initialize();
        return self::$directories;
    }
    /**
     * @throws Exception
     */
    public function isExcluded(string $file) : bool
    {
        if (\defined('PHPUNIT_TESTSUITE')) {
            return \false;
        }
        $this->initialize();
        foreach (self::$directories as $directory) {
            if (\strpos($file, $directory) === 0) {
                return \true;
            }
        }
        return \false;
    }
    /**
     * @throws Exception
     */
    private function initialize() : void
    {
        if (self::$directories === null) {
            self::$directories = [];
            foreach (self::EXCLUDED_CLASS_NAMES as $className => $parent) {
                if (!\class_exists($className)) {
                    continue;
                }
                try {
                    $directory = (new \ReflectionClass($className))->getFileName();
                    // @codeCoverageIgnoreStart
                } catch (\ReflectionException $e) {
                    throw new \MailerLite_Woo\PHPUnit\Util\Exception($e->getMessage(), (int) $e->getCode(), $e);
                }
                // @codeCoverageIgnoreEnd
                for ($i = 0; $i < $parent; $i++) {
                    $directory = \dirname($directory);
                }
                self::$directories[] = $directory;
            }
            // Hide process isolation workaround on Windows.
            if (\DIRECTORY_SEPARATOR === '\\') {
                // tempnam() prefix is limited to first 3 chars.
                // @see https://php.net/manual/en/function.tempnam.php
                self::$directories[] = \sys_get_temp_dir() . '\\PHP';
            }
        }
    }
}
