<?php

declare (strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\PHPUnit\Util\TestDox;

use function array_filter;
use function get_class;
use function implode;
use function strpos;
use DOMDocument;
use DOMElement;
use MailerLite_Woo\PHPUnit\Framework\AssertionFailedError;
use MailerLite_Woo\PHPUnit\Framework\Exception;
use MailerLite_Woo\PHPUnit\Framework\Test;
use MailerLite_Woo\PHPUnit\Framework\TestCase;
use MailerLite_Woo\PHPUnit\Framework\TestListener;
use MailerLite_Woo\PHPUnit\Framework\TestSuite;
use MailerLite_Woo\PHPUnit\Framework\Warning;
use MailerLite_Woo\PHPUnit\Util\Printer;
use MailerLite_Woo\PHPUnit\Util\Test as TestUtil;
use ReflectionClass;
use ReflectionException;
use Throwable;
/**
 * @internal This class is not covered by the backward compatibility promise for PHPUnit
 */
final class XmlResultPrinter extends \MailerLite_Woo\PHPUnit\Util\Printer implements \MailerLite_Woo\PHPUnit\Framework\TestListener
{
    /**
     * @var DOMDocument
     */
    private $document;
    /**
     * @var DOMElement
     */
    private $root;
    /**
     * @var NamePrettifier
     */
    private $prettifier;
    /**
     * @var null|Throwable
     */
    private $exception;
    /**
     * @param resource|string $out
     *
     * @throws Exception
     */
    public function __construct($out = null)
    {
        $this->document = new \DOMDocument('1.0', 'UTF-8');
        $this->document->formatOutput = \true;
        $this->root = $this->document->createElement('tests');
        $this->document->appendChild($this->root);
        $this->prettifier = new \MailerLite_Woo\PHPUnit\Util\TestDox\NamePrettifier();
        parent::__construct($out);
    }
    /**
     * Flush buffer and close output.
     */
    public function flush() : void
    {
        $this->write($this->document->saveXML());
        parent::flush();
    }
    /**
     * An error occurred.
     */
    public function addError(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
        $this->exception = $t;
    }
    /**
     * A warning occurred.
     */
    public function addWarning(\MailerLite_Woo\PHPUnit\Framework\Test $test, \MailerLite_Woo\PHPUnit\Framework\Warning $e, float $time) : void
    {
    }
    /**
     * A failure occurred.
     */
    public function addFailure(\MailerLite_Woo\PHPUnit\Framework\Test $test, \MailerLite_Woo\PHPUnit\Framework\AssertionFailedError $e, float $time) : void
    {
        $this->exception = $e;
    }
    /**
     * Incomplete test.
     */
    public function addIncompleteTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
    }
    /**
     * Risky test.
     */
    public function addRiskyTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
    }
    /**
     * Skipped test.
     */
    public function addSkippedTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, \Throwable $t, float $time) : void
    {
    }
    /**
     * A test suite started.
     */
    public function startTestSuite(\MailerLite_Woo\PHPUnit\Framework\TestSuite $suite) : void
    {
    }
    /**
     * A test suite ended.
     */
    public function endTestSuite(\MailerLite_Woo\PHPUnit\Framework\TestSuite $suite) : void
    {
    }
    /**
     * A test started.
     */
    public function startTest(\MailerLite_Woo\PHPUnit\Framework\Test $test) : void
    {
        $this->exception = null;
    }
    /**
     * A test ended.
     *
     * @throws \SebastianBergmann\RecursionContext\InvalidArgumentException
     */
    public function endTest(\MailerLite_Woo\PHPUnit\Framework\Test $test, float $time) : void
    {
        if (!$test instanceof \MailerLite_Woo\PHPUnit\Framework\TestCase) {
            return;
        }
        $groups = \array_filter($test->getGroups(), static function ($group) {
            return !($group === 'small' || $group === 'medium' || $group === 'large' || \strpos($group, '__phpunit_') === 0);
        });
        $testNode = $this->document->createElement('test');
        $testNode->setAttribute('className', \get_class($test));
        $testNode->setAttribute('methodName', $test->getName());
        $testNode->setAttribute('prettifiedClassName', $this->prettifier->prettifyTestClass(\get_class($test)));
        $testNode->setAttribute('prettifiedMethodName', $this->prettifier->prettifyTestCase($test));
        $testNode->setAttribute('status', (string) $test->getStatus());
        $testNode->setAttribute('time', (string) $time);
        $testNode->setAttribute('size', (string) $test->getSize());
        $testNode->setAttribute('groups', \implode(',', $groups));
        foreach ($groups as $group) {
            $groupNode = $this->document->createElement('group');
            $groupNode->setAttribute('name', $group);
            $testNode->appendChild($groupNode);
        }
        $annotations = \MailerLite_Woo\PHPUnit\Util\Test::parseTestMethodAnnotations(\get_class($test), $test->getName(\false));
        foreach (['class', 'method'] as $type) {
            foreach ($annotations[$type] as $annotation => $values) {
                if ($annotation !== 'covers' && $annotation !== 'uses') {
                    continue;
                }
                foreach ($values as $value) {
                    $coversNode = $this->document->createElement($annotation);
                    $coversNode->setAttribute('target', $value);
                    $testNode->appendChild($coversNode);
                }
            }
        }
        foreach ($test->doubledTypes() as $doubledType) {
            $testDoubleNode = $this->document->createElement('testDouble');
            $testDoubleNode->setAttribute('type', $doubledType);
            $testNode->appendChild($testDoubleNode);
        }
        $inlineAnnotations = \MailerLite_Woo\PHPUnit\Util\Test::getInlineAnnotations(\get_class($test), $test->getName(\false));
        if (isset($inlineAnnotations['given'], $inlineAnnotations['when'], $inlineAnnotations['then'])) {
            $testNode->setAttribute('given', $inlineAnnotations['given']['value']);
            $testNode->setAttribute('givenStartLine', (string) $inlineAnnotations['given']['line']);
            $testNode->setAttribute('when', $inlineAnnotations['when']['value']);
            $testNode->setAttribute('whenStartLine', (string) $inlineAnnotations['when']['line']);
            $testNode->setAttribute('then', $inlineAnnotations['then']['value']);
            $testNode->setAttribute('thenStartLine', (string) $inlineAnnotations['then']['line']);
        }
        if ($this->exception !== null) {
            if ($this->exception instanceof \MailerLite_Woo\PHPUnit\Framework\Exception) {
                $steps = $this->exception->getSerializableTrace();
            } else {
                $steps = $this->exception->getTrace();
            }
            try {
                $file = (new \ReflectionClass($test))->getFileName();
                // @codeCoverageIgnoreStart
            } catch (\ReflectionException $e) {
                throw new \MailerLite_Woo\PHPUnit\Framework\Exception($e->getMessage(), (int) $e->getCode(), $e);
            }
            // @codeCoverageIgnoreEnd
            foreach ($steps as $step) {
                if (isset($step['file']) && $step['file'] === $file) {
                    $testNode->setAttribute('exceptionLine', (string) $step['line']);
                    break;
                }
            }
            $testNode->setAttribute('exceptionMessage', $this->exception->getMessage());
        }
        $this->root->appendChild($testNode);
    }
}
