<?php

declare (strict_types=1);
/*
 * This file is part of sebastian/complexity.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\SebastianBergmann\Complexity;

use function assert;
use function is_array;
use MailerLite_Woo\PhpParser\Node;
use MailerLite_Woo\PhpParser\Node\Name;
use MailerLite_Woo\PhpParser\Node\Stmt;
use MailerLite_Woo\PhpParser\Node\Stmt\Class_;
use MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod;
use MailerLite_Woo\PhpParser\Node\Stmt\Function_;
use MailerLite_Woo\PhpParser\Node\Stmt\Trait_;
use MailerLite_Woo\PhpParser\NodeTraverser;
use MailerLite_Woo\PhpParser\NodeVisitorAbstract;
final class ComplexityCalculatingVisitor extends \MailerLite_Woo\PhpParser\NodeVisitorAbstract
{
    /**
     * @psalm-var list<Complexity>
     */
    private $result = [];
    /**
     * @var bool
     */
    private $shortCircuitTraversal;
    public function __construct(bool $shortCircuitTraversal)
    {
        $this->shortCircuitTraversal = $shortCircuitTraversal;
    }
    public function enterNode(\MailerLite_Woo\PhpParser\Node $node) : ?int
    {
        if (!$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod && !$node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Function_) {
            return null;
        }
        if ($node instanceof \MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod) {
            $name = $this->classMethodName($node);
        } else {
            $name = $this->functionName($node);
        }
        $statements = $node->getStmts();
        \assert(\is_array($statements));
        $this->result[] = new \MailerLite_Woo\SebastianBergmann\Complexity\Complexity($name, $this->cyclomaticComplexity($statements));
        if ($this->shortCircuitTraversal) {
            return \MailerLite_Woo\PhpParser\NodeTraverser::DONT_TRAVERSE_CHILDREN;
        }
        return null;
    }
    public function result() : \MailerLite_Woo\SebastianBergmann\Complexity\ComplexityCollection
    {
        return \MailerLite_Woo\SebastianBergmann\Complexity\ComplexityCollection::fromList(...$this->result);
    }
    /**
     * @param Stmt[] $statements
     */
    private function cyclomaticComplexity(array $statements) : int
    {
        $traverser = new \MailerLite_Woo\PhpParser\NodeTraverser();
        $cyclomaticComplexityCalculatingVisitor = new \MailerLite_Woo\SebastianBergmann\Complexity\CyclomaticComplexityCalculatingVisitor();
        $traverser->addVisitor($cyclomaticComplexityCalculatingVisitor);
        /* @noinspection UnusedFunctionResultInspection */
        $traverser->traverse($statements);
        return $cyclomaticComplexityCalculatingVisitor->cyclomaticComplexity();
    }
    private function classMethodName(\MailerLite_Woo\PhpParser\Node\Stmt\ClassMethod $node) : string
    {
        $parent = $node->getAttribute('parent');
        \assert($parent instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Class_ || $parent instanceof \MailerLite_Woo\PhpParser\Node\Stmt\Trait_);
        \assert(isset($parent->namespacedName));
        \assert($parent->namespacedName instanceof \MailerLite_Woo\PhpParser\Node\Name);
        return $parent->namespacedName->toString() . '::' . $node->name->toString();
    }
    private function functionName(\MailerLite_Woo\PhpParser\Node\Stmt\Function_ $node) : string
    {
        \assert(isset($node->namespacedName));
        \assert($node->namespacedName instanceof \MailerLite_Woo\PhpParser\Node\Name);
        return $node->namespacedName->toString();
    }
}
