<?php

declare (strict_types=1);
/*
 * This file is part of sebastian/type.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace MailerLite_Woo\SebastianBergmann\Type;

use function count;
use function implode;
use function sort;
final class UnionType extends \MailerLite_Woo\SebastianBergmann\Type\Type
{
    /**
     * @psalm-var list<Type>
     */
    private $types;
    /**
     * @throws RuntimeException
     */
    public function __construct(\MailerLite_Woo\SebastianBergmann\Type\Type ...$types)
    {
        $this->ensureMinimumOfTwoTypes(...$types);
        $this->ensureOnlyValidTypes(...$types);
        $this->types = $types;
    }
    public function isAssignable(\MailerLite_Woo\SebastianBergmann\Type\Type $other) : bool
    {
        foreach ($this->types as $type) {
            if ($type->isAssignable($other)) {
                return \true;
            }
        }
        return \false;
    }
    public function asString() : string
    {
        return $this->name();
    }
    /**
     * @deprecated
     *
     * @codeCoverageIgnore
     */
    public function getReturnTypeDeclaration() : string
    {
        return ': ' . $this->name();
    }
    public function name() : string
    {
        $types = [];
        foreach ($this->types as $type) {
            $types[] = $type->name();
        }
        \sort($types);
        return \implode('|', $types);
    }
    public function allowsNull() : bool
    {
        foreach ($this->types as $type) {
            if ($type instanceof \MailerLite_Woo\SebastianBergmann\Type\NullType) {
                return \true;
            }
        }
        return \false;
    }
    /**
     * @throws RuntimeException
     */
    private function ensureMinimumOfTwoTypes(\MailerLite_Woo\SebastianBergmann\Type\Type ...$types) : void
    {
        if (\count($types) < 2) {
            throw new \MailerLite_Woo\SebastianBergmann\Type\RuntimeException('A union type must be composed of at least two types');
        }
    }
    /**
     * @throws RuntimeException
     */
    private function ensureOnlyValidTypes(\MailerLite_Woo\SebastianBergmann\Type\Type ...$types) : void
    {
        foreach ($types as $type) {
            if ($type instanceof \MailerLite_Woo\SebastianBergmann\Type\UnknownType) {
                throw new \MailerLite_Woo\SebastianBergmann\Type\RuntimeException('A union type must not be composed of an unknown type');
            }
            if ($type instanceof \MailerLite_Woo\SebastianBergmann\Type\VoidType) {
                throw new \MailerLite_Woo\SebastianBergmann\Type\RuntimeException('A union type must not be composed of a void type');
            }
        }
    }
}
